<?php

declare (strict_types = 1);

namespace App\Nova\Lenses;

use App\Models\Product;
use App\Models\Retailer;
use DB;
use Laravel\Nova\Fields\Select;
use Laravel\Nova\Fields\Text;
use Laravel\Nova\Http\Requests\LensRequest;
use Laravel\Nova\Http\Requests\NovaRequest;
use Laravel\Nova\Lenses\Lens;

class CompareProduct extends Lens
{

    public static function query(LensRequest $request, $query)
    {
        $data = self::getDataFromUrl();
        if ($data) {
            $existFields = Product::where('retailer_id', $data['retailer'])->pluck('sku')->all();

            return $query
                ->where('retailer_id', '!=', $data['retailer'])
                ->whereIn('sku', $existFields)
                ->join('retailers as r', 'products.retailer_id', '=', 'r.id')
                ->select(
                    'sku',
                    DB::raw('group_concat(products.id) as id'),
                    DB::raw('group_concat(' . $data['field'] . ') as ' . $data['field']),
                    DB::raw('group_concat(products.name) as name'),
                    DB::raw('group_concat(r.id) as retailer_id'))
                ->groupBy('sku');
        }
    }

    /**
     * Get the fields available to the lens.
     *
     * @param  \Laravel\Nova\Http\Requests\NovaRequest  $request
     * @return array
     */

    public function fields(NovaRequest $request)
    {
        if (self::getDataFromUrl()) {
            $data = self::getDataFromUrl();
            $fields = [
                Text::make('Name', function () use ($data) {
                    $sku = Product::where('retailer_id', $this['retailer_id'])->where('sku', $this['sku'])->pluck('sku')->first();
                    return Product::where('sku', $sku)->where('retailer_id', $data['retailer'])->pluck('name')->first();
                })
                    ->sortable()
                    ->rules('required', 'max:255', 'min:2'),

                Text::make('SKU')
                    ->sortable()
                    ->rules('numeric'),
            ];
            $existFields = Product::where('retailer_id', $data['retailer'])->pluck('sku')->all();
            $retailers = Retailer::where('id', '!=', $data['retailer'])->pluck('id', 'name')->all();
            $count = 0;
            foreach ($retailers as $retailer => $key) {
                $availeble = Product::where('retailer_id', $key)->whereIn('sku', $existFields)->get();
                if (count($availeble) !== 0) {
                    array_push($fields,
                        Text::make($retailer . ' ' . $data['field'], function () use ($count, $data) {
                            $field = $data['field'];
                            $fields = explode(',', $this->$field);
                            if (count($fields) < $count + 1) {
                                return null;
                            }
                            return $fields[$count];
                        }));
                }
                $count++;
            }
            return $fields;
        }
        return [];
    }

    /**
     * Get the cards available on the lens.
     *
     * @param  \Laravel\Nova\Http\Requests\NovaRequest  $request
     * @return array
     */
    public function cards(NovaRequest $request)
    {
        return [];
    }

    /**
     * Get the filters available for the lens.
     *
     * @param  \Laravel\Nova\Http\Requests\NovaRequest  $request
     * @return array
     */
    public function filters(NovaRequest $request)
    {
        return [];
    }

    /**
     * Get the actions available on the lens.
     *
     * @param  \Laravel\Nova\Http\Requests\NovaRequest  $request
     * @return array
     */
    public function actions(NovaRequest $request)
    {
        if (self::getDataFromUrl()) {
            $data = self::getDataFromUrl();
            $retailers = self::getRetailers();
            return [
                \App\Nova\Actions\ChangeValue::make($data['field'], $data['retailer'], $retailers),
            ];
        }
    }

    /**
     * Get the URI key for the lens.
     *
     * @return string
     */
    public function uriKey()
    {
        return 'compare-product';
    }

    public static function getDataFromUrl()
    {
        if (!array_key_exists('query', parse_url(url()->previous()))) {
            return false;
        }
        parse_str(parse_url(url()->previous())['query'], $data);
        return $data;
    }

    public static function getRetailers()
    {
        $data = self::getDataFromUrl();
        $trueRetailers = [];
        $existFields = Product::where('retailer_id', $data['retailer'])->pluck('sku')->all();
        $retailers = Retailer::where('id', '!=', $data['retailer'])->pluck('id', 'name')->all();
        $count = 0;
        foreach ($retailers as $retailer => $key) {
            $availeble = Product::where('retailer_id', $key)->whereIn('sku', $existFields)->get();
            if (count($availeble) !== 0) {
                array_push($trueRetailers, $key);
            }
            $count++;
        }
        return $trueRetailers;
    }
}
